@@ -30,5 +30,8 @@ extern void ifx_audio_settings_exit();
 extern void ifx_radio_settings_init();
 extern void ifx_radio_settings_exit();
 
+extern void ifx_gprs_context_init();
+extern void ifx_gprs_context_exit();
+
 extern void ifx_stk_init();
 extern void ifx_stk_exit();
