@@ -46,7 +46,7 @@ struct stk_data {
 
 static const char *none_prefix[] = { NULL };
 static const char *sate_prefix[] = { "+SATE:", NULL };
-static const char *stkprof_prefix[] = { "+STKPROF:", NULL };
+static const char *cfun_prefix[] = { "+CFUN:", NULL };
 
 static void sate_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
@@ -197,24 +197,26 @@ static void sati_notify(GAtResult *result, gpointer user_data)
 
 static void satn_notify(GAtResult *result, gpointer user_data)
 {
+	struct ofono_stk *stk = user_data;
 	GAtResultIter iter;
 	const guint8 *pdu;
 	gint len;
 
 	DBG("");
 
+	/* Proactive command has been handled by the modem. */
+	g_at_result_iter_init(&iter, result);
+
 	if (g_at_result_iter_next(&iter, "+SATN:") == FALSE)
 		return;
 
 	if (g_at_result_iter_next_hexstring(&iter, &pdu, &len) == FALSE)
 		return;
 
-	DBG("len %d", len);
+	if (len == 0)
+		return;
 
-	/* Proactive command has been handled by the modem.  If the
-	 * command was for Setup Call then a response with AT+SATD
-	 * is required.  This is not handled properly yet.
-	 */
+	ofono_stk_proactive_command_handled_notify(stk, len, pdu);
 }
 
 static void satf_notify(GAtResult *result, gpointer user_data)
@@ -238,10 +240,11 @@ static void satf_notify(GAtResult *result, gpointer user_data)
 
 	DBG("sw1 %d sw2 %d", sw1, sw2);
 
-	ofono_stk_proactive_session_end_notify(stk);
+	if (sw1 == 0x90 && sw2 == 0x00)
+		ofono_stk_proactive_session_end_notify(stk);
 }
 
-static void stkprof_support_cb(gboolean ok, GAtResult *result,
+static void cfun_support_cb(gboolean ok, GAtResult *result,
 						gpointer user_data)
 {
 	struct ofono_stk *stk = user_data;
@@ -256,11 +259,8 @@ static void stkprof_support_cb(gboolean ok, GAtResult *result,
 	g_at_chat_register(sd->chat, "+SATN:", satn_notify, FALSE, stk, NULL);
 	g_at_chat_register(sd->chat, "+SATF:", satf_notify, FALSE, stk, NULL);
 
-	g_at_chat_send(sd->chat, "AT+STKPROF?", stkprof_prefix,
-							NULL, NULL, NULL);
-
-	g_at_chat_send(sd->chat, "AT+STKPROF=4,\"1F7F\"", none_prefix,
-							NULL, NULL, NULL);
+	g_at_chat_send(sd->chat, "AT+CFUN=6", none_prefix,
+						NULL, NULL, NULL);
 
 	ofono_stk_register(stk);
 }
@@ -280,8 +280,8 @@ static int ifx_stk_probe(struct ofono_stk *stk, unsigned int vendor, void *data)
 
 	ofono_stk_set_data(stk, sd);
 
-	g_at_chat_send(sd->chat, "AT+STKPROF=?", stkprof_prefix,
-					stkprof_support_cb, stk, NULL);
+	g_at_chat_send(sd->chat, "AT+CFUN=?", cfun_prefix,
+					cfun_support_cb, stk, NULL);
 
 	return 0;
 }
